"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = void 0;
const fs = require("fs");
const path = require("path");
const ts = require("typescript/lib/tsserverlibrary");
function noop() { }
function nowString() {
    // E.g. "12:34:56.789"
    const d = new Date();
    return `${d.getHours()}:${d.getMinutes()}:${d.getSeconds()}.${d.getMilliseconds()}`;
}
class Logger {
    constructor(traceToConsole, level, logFilename) {
        this.traceToConsole = traceToConsole;
        this.level = level;
        this.logFilename = logFilename;
        this.fd = -1;
        this.seq = 0;
        this.inGroup = false;
        this.firstInGroup = true;
        if (logFilename) {
            try {
                const dir = path.dirname(logFilename);
                if (!fs.existsSync(dir)) {
                    fs.mkdirSync(dir);
                }
                this.fd = fs.openSync(logFilename, 'w');
            }
            catch (_a) {
                // swallow the error and keep logging disabled if file cannot be opened
            }
        }
    }
    static padStringRight(str, padding) {
        return (str + padding).slice(0, padding.length);
    }
    close() {
        if (this.fd >= 0) {
            fs.close(this.fd, noop);
        }
    }
    getLogFileName() {
        return this.logFilename;
    }
    perftrc(s) {
        this.msg(s, ts.server.Msg.Perf);
    }
    info(s) {
        this.msg(s, ts.server.Msg.Info);
    }
    err(s) {
        this.msg(s, ts.server.Msg.Err);
    }
    startGroup() {
        this.inGroup = true;
        this.firstInGroup = true;
    }
    endGroup() {
        this.inGroup = false;
    }
    loggingEnabled() {
        return !!this.logFilename || this.traceToConsole;
    }
    hasLevel(level) {
        return this.loggingEnabled() && this.level >= level;
    }
    msg(s, type = ts.server.Msg.Err) {
        if (!this.canWrite) {
            return;
        }
        s = `[${nowString()}] ${s}\n`;
        if (!this.inGroup || this.firstInGroup) {
            const prefix = Logger.padStringRight(type + ' ' + this.seq.toString(), '          ');
            s = prefix + s;
        }
        this.write(s);
        if (!this.inGroup) {
            this.seq++;
        }
    }
    get canWrite() {
        return this.fd >= 0 || this.traceToConsole;
    }
    write(s) {
        if (this.fd >= 0) {
            const buf = Buffer.from(s);
            fs.writeSync(this.fd, buf, 0, buf.length, /*position*/ null);
        }
        if (this.traceToConsole) {
            console.warn(s);
        }
    }
}
exports.Logger = Logger;
